@@ -41,22 +41,26 @@ class Session extends EventEmitter {
   }
 
   post(method, params, callback) {
-    if (typeof method !== 'string')
+    if (typeof method !== 'string') {
       throw new TypeError(
-          `"method" must be a string, got ${typeof method} instead`);
+        `"method" must be a string, got ${typeof method} instead`);
+    }
     if (!callback && util.isFunction(params)) {
       callback = params;
       params = null;
     }
-    if (params && typeof params !== 'object')
+    if (params && typeof params !== 'object') {
       throw new TypeError(
-          `"params" must be an object, got ${typeof params} instead`);
-    if (callback && typeof callback !== 'function')
+        `"params" must be an object, got ${typeof params} instead`);
+    }
+    if (callback && typeof callback !== 'function') {
       throw new TypeError(
-          `"callback" must be a function, got ${typeof callback} instead`);
+        `"callback" must be a function, got ${typeof callback} instead`);
+    }
 
-    if (!this[connectionSymbol])
+    if (!this[connectionSymbol]) {
       throw new Error('Session is not connected');
+    }
     const id = this[nextIdSymbol]++;
     const message = {id, method};
     if (params) {
