@@ -43,10 +43,12 @@ function setup_cpuUsage() {
     }
 
     // If a previous value was passed in, return diff of current from previous.
-    if (prevValue) return {
-      user: cpuValues[0] - prevValue.user,
-      system: cpuValues[1] - prevValue.system
-    };
+    if (prevValue) {
+      return {
+        user: cpuValues[0] - prevValue.user,
+        system: cpuValues[1] - prevValue.system
+      };
+    }
 
     // If no previous value passed in, return current value.
     return {
@@ -153,8 +155,6 @@ function setupKillAndExit() {
       process._exiting = true;
       process.emit('exit', process.exitCode || 0);
     }
-
-    require('trace_mgr').onExitHandler(process.exitCode || 0); //ENABLE_TTD
     process.reallyExit(process.exitCode || 0);
   };
 
@@ -187,7 +187,7 @@ function setupKillAndExit() {
   };
 }
 
-var ttdSigIntHandler = undefined;
+
 function setupSignalHandlers() {
   // Load events module in order to access prototype elements on process like
   // process.addListener.
@@ -206,14 +206,7 @@ function setupSignalHandlers() {
 
       wrap.unref();
 
-      if (type === 'SIGINT' && global.enabledDiagnosticsTrace) {
-        //TODO: this is probably a little over complicated
-        //  -- should unify this and ttdSigIntHandler setup below
-        //  -- ttdSigIntHandler will emit event after it runs
-        wrap.onsignal = function() { }; //ENABLE_TTD
-      } else {
-        wrap.onsignal = function() { process.emit(type); };
-      }
+      wrap.onsignal = function() { process.emit(type); };
 
       const signum = lazyConstants()[type];
       const err = wrap.start(signum);
@@ -233,34 +226,6 @@ function setupSignalHandlers() {
       delete signalWraps[type];
     }
   });
-
-  //ENABLE_TTD
-  if (global.enabledDiagnosticsTrace) {
-    (function() {
-      var Signal = process.binding('signal_wrap').Signal;
-      var wrap = new Signal();
-
-      wrap.unref();
-
-      var handler = require('trace_mgr').onSigIntHandler;
-      wrap.onsignal = function() {
-        handler(signalWraps.hasOwnProperty('SIGINT'));
-        process.emit('SIGINT');
-      };
-
-      const signum = lazyConstants()['SIGINT'];
-      const err = wrap.start(signum);
-      if (err) {
-        wrap.close();
-        const errnoException = require('util')._errnoException;
-        throw errnoException(err, 'uv_signal_start');
-      }
-
-      if (!ttdSigIntHandler) {
-        ttdSigIntHandler = wrap;
-      }
-    })();
-  }
 }
 
 
