@@ -36,6 +36,7 @@ using v8::Context;
 using v8::Float64Array;
 using v8::Function;
 using v8::FunctionCallbackInfo;
+using v8::FunctionTemplate;
 using v8::HandleScope;
 using v8::HeapProfiler;
 using v8::Integer;
@@ -44,8 +45,10 @@ using v8::Local;
 using v8::MaybeLocal;
 using v8::Number;
 using v8::Object;
+using v8::ObjectTemplate;
 using v8::Promise;
 using v8::PromiseHookType;
+using v8::PropertyCallbackInfo;
 using v8::RetainedObjectInfo;
 using v8::String;
 using v8::Symbol;
@@ -282,45 +285,104 @@ bool AsyncWrap::EmitAfter(Environment* env, double async_id) {
 class PromiseWrap : public AsyncWrap {
  public:
   PromiseWrap(Environment* env, Local<Object> object, bool silent)
-    : AsyncWrap(env, object, PROVIDER_PROMISE, silent) {}
+    : AsyncWrap(env, object, PROVIDER_PROMISE, silent) {
+    MakeWeak(this);
+  }
   size_t self_size() const override { return sizeof(*this); }
+
+  static constexpr int kPromiseField = 1;
+  static constexpr int kParentIdField = 2;
+  static constexpr int kInternalFieldCount = 3;
+
+  static PromiseWrap* New(Environment* env,
+                          Local<Promise> promise,
+                          PromiseWrap* parent_wrap,
+                          bool silent);
+  static void GetPromise(Local<String> property,
+                         const PropertyCallbackInfo<Value>& info);
+  static void GetParentId(Local<String> property,
+                          const PropertyCallbackInfo<Value>& info);
 };
 
+PromiseWrap* PromiseWrap::New(Environment* env,
+                              Local<Promise> promise,
+                              PromiseWrap* parent_wrap,
+                              bool silent) {
+  Local<Object> object = env->promise_wrap_template()
+                            ->NewInstance(env->context()).ToLocalChecked();
+  object->SetInternalField(PromiseWrap::kPromiseField, promise);
+  if (parent_wrap != nullptr) {
+    object->SetInternalField(PromiseWrap::kParentIdField,
+                             Number::New(env->isolate(),
+                                         parent_wrap->get_id()));
+  }
+  CHECK_EQ(promise->GetAlignedPointerFromInternalField(0), nullptr);
+  promise->SetInternalField(0, object);
+  return new PromiseWrap(env, object, silent);
+}
+
+void PromiseWrap::GetPromise(Local<String> property,
+                             const PropertyCallbackInfo<Value>& info) {
+  info.GetReturnValue().Set(info.Holder()->GetInternalField(kPromiseField));
+}
+
+void PromiseWrap::GetParentId(Local<String> property,
+                              const PropertyCallbackInfo<Value>& info) {
+  info.GetReturnValue().Set(info.Holder()->GetInternalField(kParentIdField));
+}
 
 static void PromiseHook(PromiseHookType type, Local<Promise> promise,
                         Local<Value> parent, void* arg) {
-  Local<Context> context = promise->CreationContext();
-  Environment* env = Environment::GetCurrent(context);
-  PromiseWrap* wrap = Unwrap<PromiseWrap>(promise);
+  Environment* env = static_cast<Environment*>(arg);
+  Local<Value> resource_object_value = promise->GetInternalField(0);
+  PromiseWrap* wrap = nullptr;
+  if (resource_object_value->IsObject()) {
+    Local<Object> resource_object = resource_object_value.As<Object>();
+    wrap = Unwrap<PromiseWrap>(resource_object);
+  }
+
   if (type == PromiseHookType::kInit || wrap == nullptr) {
     bool silent = type != PromiseHookType::kInit;
-    // set parent promise's async Id as this promise's triggerId
+    PromiseWrap* parent_wrap = nullptr;
+
+    // set parent promise's async Id as this promise's triggerAsyncId
     if (parent->IsPromise()) {
       // parent promise exists, current promise
       // is a chained promise, so we set parent promise's id as
-      // current promise's triggerId
+      // current promise's triggerAsyncId
       Local<Promise> parent_promise = parent.As<Promise>();
-      auto parent_wrap = Unwrap<PromiseWrap>(parent_promise);
+      Local<Value> parent_resource = parent_promise->GetInternalField(0);
+      if (parent_resource->IsObject()) {
+        parent_wrap = Unwrap<PromiseWrap>(parent_resource.As<Object>());
+      }
 
       if (parent_wrap == nullptr) {
-        // create a new PromiseWrap for parent promise with silent parameter
-        parent_wrap = new PromiseWrap(env, parent_promise, true);
-        parent_wrap->MakeWeak(parent_wrap);
+        parent_wrap = PromiseWrap::New(env, parent_promise, nullptr, true);
       }
       // get id from parentWrap
       double trigger_id = parent_wrap->get_id();
       env->set_init_trigger_id(trigger_id);
     }
-    wrap = new PromiseWrap(env, promise, silent);
-    wrap->MakeWeak(wrap);
+
+    wrap = PromiseWrap::New(env, promise, parent_wrap, silent);
   } else if (type == PromiseHookType::kResolve) {
     // TODO(matthewloring): need to expose this through the async hooks api.
   }
+
   CHECK_NE(wrap, nullptr);
   if (type == PromiseHookType::kBefore) {
+    env->async_hooks()->push_ids(wrap->get_id(), wrap->get_trigger_id());
     PreCallbackExecution(wrap, false);
   } else if (type == PromiseHookType::kAfter) {
     PostCallbackExecution(wrap, false);
+    if (env->current_async_id() == wrap->get_id()) {
+      // This condition might not be true if async_hooks was enabled during
+      // the promise callback execution.
+      // Popping it off the stack can be skipped in that case, because is is
+      // known that it would correspond to exactly one call with
+      // PromiseHookType::kBefore that was not witnessed by the PromiseHook.
+      env->async_hooks()->pop_ids(wrap->get_id());
+    }
   }
 }
 
@@ -349,8 +411,41 @@ static void SetupHooks(const FunctionCallbackInfo<Value>& args) {
   SET_HOOK_FN(before);
   SET_HOOK_FN(after);
   SET_HOOK_FN(destroy);
-  env->AddPromiseHook(PromiseHook, nullptr);
 #undef SET_HOOK_FN
+
+  {
+    Local<FunctionTemplate> ctor =
+        FunctionTemplate::New(env->isolate());
+    ctor->SetClassName(FIXED_ONE_BYTE_STRING(env->isolate(), "PromiseWrap"));
+    Local<ObjectTemplate> promise_wrap_template = ctor->InstanceTemplate();
+    promise_wrap_template->SetInternalFieldCount(
+        PromiseWrap::kInternalFieldCount);
+    promise_wrap_template->SetAccessor(
+        FIXED_ONE_BYTE_STRING(env->isolate(), "promise"),
+        PromiseWrap::GetPromise);
+    promise_wrap_template->SetAccessor(
+        FIXED_ONE_BYTE_STRING(env->isolate(), "parentId"),
+        PromiseWrap::GetParentId);
+    env->set_promise_wrap_template(promise_wrap_template);
+  }
+}
+
+
+static void EnablePromiseHook(const FunctionCallbackInfo<Value>& args) {
+  Environment* env = Environment::GetCurrent(args);
+  env->AddPromiseHook(PromiseHook, static_cast<void*>(env));
+}
+
+
+static void DisablePromiseHook(const FunctionCallbackInfo<Value>& args) {
+  Environment* env = Environment::GetCurrent(args);
+
+  // Delay the call to `RemovePromiseHook` because we might currently be
+  // between the `before` and `after` calls of a Promise.
+  env->isolate()->EnqueueMicrotask([](void* data) {
+    Environment* env = static_cast<Environment*>(data);
+    env->RemovePromiseHook(PromiseHook, data);
+  }, static_cast<void*>(env));
 }
 
 
@@ -410,6 +505,8 @@ void AsyncWrap::Initialize(Local<Object> target,
   env->SetMethod(target, "popAsyncIds", PopAsyncIds);
   env->SetMethod(target, "clearIdStack", ClearIdStack);
   env->SetMethod(target, "addIdToDestroyList", QueueDestroyId);
+  env->SetMethod(target, "enablePromiseHook", EnablePromiseHook);
+  env->SetMethod(target, "disablePromiseHook", DisablePromiseHook);
 
   v8::PropertyAttribute ReadOnlyDontDelete =
       static_cast<v8::PropertyAttribute>(v8::ReadOnly | v8::DontDelete);
@@ -447,18 +544,9 @@ void AsyncWrap::Initialize(Local<Object> target,
       isolate,
       uid_fields_ptr,
       uid_fields_count * sizeof(*uid_fields_ptr));
-  Local<Float64Array> farray = Float64Array::New(uid_fields_ab, 0,
-                                                 uid_fields_count);
   FORCE_SET_TARGET_FIELD(target,
                          "async_uid_fields",
-                         farray);
-
-#if ENABLE_TTD_NODE
-  if (s_doTTRecord || s_doTTReplay) {
-    JsTTDNotifyLongLivedReferenceAdd(*farray);
-    env->async_hooks()->uid_fields_ttdRef = *farray;
-  }
-#endif
+                         Float64Array::New(uid_fields_ab, 0, uid_fields_count));
 
   Local<Object> constants = Object::New(isolate);
 #define SET_HOOKS_CONSTANT(name)                                              \
@@ -469,6 +557,7 @@ void AsyncWrap::Initialize(Local<Object> target,
   SET_HOOKS_CONSTANT(kBefore);
   SET_HOOKS_CONSTANT(kAfter);
   SET_HOOKS_CONSTANT(kDestroy);
+  SET_HOOKS_CONSTANT(kTotals);
   SET_HOOKS_CONSTANT(kCurrentAsyncId);
   SET_HOOKS_CONSTANT(kCurrentTriggerId);
   SET_HOOKS_CONSTANT(kAsyncUidCntr);
@@ -545,9 +634,10 @@ void AsyncWrap::AsyncReset(bool silent) {
 
   if (silent) return;
 
-  EmitAsyncInit(env(), object(),
-                env()->async_hooks()->provider_string(provider_type()),
-                async_id_, trigger_id_);
+  AsyncWrap::EmitAsyncInit(
+      env(), object(),
+      env()->async_hooks()->provider_string(provider_type()),
+      async_id_, trigger_id_);
 }
 
 
@@ -652,34 +742,106 @@ Local<Value> AsyncWrap::MakeCallback(const Local<Function> cb,
 /* Public C++ embedder API */
 
 
-async_uid AsyncHooksGetCurrentId(Isolate* isolate) {
+async_id AsyncHooksGetExecutionAsyncId(Isolate* isolate) {
   return Environment::GetCurrent(isolate)->current_async_id();
 }
 
-
-async_uid AsyncHooksGetTriggerId(Isolate* isolate) {
-  return Environment::GetCurrent(isolate)->get_init_trigger_id();
+async_id AsyncHooksGetCurrentId(Isolate* isolate) {
+  return AsyncHooksGetExecutionAsyncId(isolate);
 }
 
 
-async_uid EmitAsyncInit(Isolate* isolate,
-                     Local<Object> resource,
-                     const char* name,
-                     async_uid trigger_id) {
-  Environment* env = Environment::GetCurrent(isolate);
-  async_uid async_id = env->new_async_id();
+async_id AsyncHooksGetTriggerAsyncId(Isolate* isolate) {
+  return Environment::GetCurrent(isolate)->trigger_id();
+}
 
+async_id AsyncHooksGetTriggerId(Isolate* isolate) {
+  return AsyncHooksGetTriggerAsyncId(isolate);
+}
+
+
+async_context EmitAsyncInit(Isolate* isolate,
+                            Local<Object> resource,
+                            const char* name,
+                            async_id trigger_async_id) {
+  Environment* env = Environment::GetCurrent(isolate);
+
+  // Initialize async context struct
+  if (trigger_async_id == -1)
+    trigger_async_id = env->get_init_trigger_id();
+
+  async_context context = {
+    env->new_async_id(),  // async_id_
+    trigger_async_id  // trigger_async_id_
+  };
+
+  // Run init hooks
   Local<String> type =
       String::NewFromUtf8(isolate, name, v8::NewStringType::kInternalized)
           .ToLocalChecked();
-  AsyncWrap::EmitAsyncInit(env, resource, type, async_id, trigger_id);
-  return async_id;
+  AsyncWrap::EmitAsyncInit(env, resource, type, context.async_id,
+                           context.trigger_async_id);
+
+  return context;
 }
 
-void EmitAsyncDestroy(Isolate* isolate, async_uid id) {
-  PushBackDestroyId(Environment::GetCurrent(isolate), id);
+void EmitAsyncDestroy(Isolate* isolate, async_context asyncContext) {
+  PushBackDestroyId(Environment::GetCurrent(isolate), asyncContext.async_id);
 }
 
 }  // namespace node
 
 NODE_MODULE_CONTEXT_AWARE_BUILTIN(async_wrap, node::AsyncWrap::Initialize)
+
+
+// Only legacy public API below this line.
+
+namespace node {
+
+MaybeLocal<Value> MakeCallback(Isolate* isolate,
+                               Local<Object> recv,
+                               Local<Function> callback,
+                               int argc,
+                               Local<Value>* argv,
+                               async_id asyncId,
+                               async_id triggerAsyncId) {
+  return MakeCallback(isolate, recv, callback, argc, argv,
+                      {asyncId, triggerAsyncId});
+}
+
+MaybeLocal<Value> MakeCallback(Isolate* isolate,
+                               Local<Object> recv,
+                               const char* method,
+                               int argc,
+                               Local<Value>* argv,
+                               async_id asyncId,
+                               async_id triggerAsyncId) {
+  return MakeCallback(isolate, recv, method, argc, argv,
+                      {asyncId, triggerAsyncId});
+}
+
+MaybeLocal<Value> MakeCallback(Isolate* isolate,
+                               Local<Object> recv,
+                               Local<String> symbol,
+                               int argc,
+                               Local<Value>* argv,
+                               async_id asyncId,
+                               async_id triggerAsyncId) {
+  return MakeCallback(isolate, recv, symbol, argc, argv,
+                      {asyncId, triggerAsyncId});
+}
+
+// Undo the Node-8.x-only alias from node.h
+#undef EmitAsyncInit
+
+async_uid EmitAsyncInit(Isolate* isolate,
+                        Local<Object> resource,
+                        const char* name,
+                        async_id trigger_async_id) {
+  return EmitAsyncInit__New(isolate,
+                            resource,
+                            name,
+                            trigger_async_id).async_id;
+}
+
+}  // namespace node
