@@ -370,7 +370,7 @@ class InspectorSocketTest : public ::testing::Test {
     sockaddr_in addr;
     uv_tcp_init(&loop, &server);
     uv_tcp_init(&loop, &client_socket);
-    uv_ip4_addr("127.0.0.1", PORT, &addr);
+    GTEST_ASSERT_EQ(0, uv_ip4_addr("127.0.0.1", PORT, &addr));
     uv_tcp_bind(&server, reinterpret_cast<const struct sockaddr*>(&addr), 0);
     GTEST_ASSERT_EQ(0, uv_listen(reinterpret_cast<uv_stream_t*>(&server),
                                  1, on_new_connection));
