@@ -1,5 +1,26 @@
+// Copyright Joyent, Inc. and other Node contributors.
+//
+// Permission is hereby granted, free of charge, to any person obtaining a
+// copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to permit
+// persons to whom the Software is furnished to do so, subject to the
+// following conditions:
+//
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
+// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
+// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
+// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
+// USE OR OTHER DEALINGS IN THE SOFTWARE.
+
 'use strict';
-require('../common');
+const common = require('../common');
 const assert = require('assert');
 const a = require('assert');
 
@@ -62,7 +83,6 @@ assert.doesNotThrow(makeBlock(a.notStrictEqual, 2, '2'),
                     'notStrictEqual(2, \'2\')');
 
 // deepEqual joy!
-// 7.2
 assert.doesNotThrow(makeBlock(a.deepEqual, new Date(2000, 3, 14),
                               new Date(2000, 3, 14)),
                     'deepEqual(new Date(2000, 3, 14), new Date(2000, 3, 14))');
@@ -84,40 +104,58 @@ assert.doesNotThrow(makeBlock(
                     'notDeepEqual(new Date(), new Date(2000, 3, 14))'
 );
 
-// 7.3
 assert.doesNotThrow(makeBlock(a.deepEqual, /a/, /a/));
 assert.doesNotThrow(makeBlock(a.deepEqual, /a/g, /a/g));
 assert.doesNotThrow(makeBlock(a.deepEqual, /a/i, /a/i));
 assert.doesNotThrow(makeBlock(a.deepEqual, /a/m, /a/m));
 assert.doesNotThrow(makeBlock(a.deepEqual, /a/igm, /a/igm));
 assert.throws(makeBlock(a.deepEqual, /ab/, /a/),
-              /^AssertionError: \/ab\/ deepEqual \/a\/$/);
+              common.expectsError({
+                code: 'ERR_ASSERTION',
+                type: a.AssertionError,
+                message: /^\/ab\/ deepEqual \/a\/$/
+              }));
 assert.throws(makeBlock(a.deepEqual, /a/g, /a/),
-              /^AssertionError: \/a\/g deepEqual \/a\/$/);
+              common.expectsError({
+                code: 'ERR_ASSERTION',
+                type: a.AssertionError,
+                message: /^\/a\/g deepEqual \/a\/$/
+              }));
 assert.throws(makeBlock(a.deepEqual, /a/i, /a/),
-              /^AssertionError: \/a\/i deepEqual \/a\/$/);
+              common.expectsError({
+                code: 'ERR_ASSERTION',
+                type: a.AssertionError,
+                message: /^\/a\/i deepEqual \/a\/$/
+              }));
 assert.throws(makeBlock(a.deepEqual, /a/m, /a/),
-              /^AssertionError: \/a\/m deepEqual \/a\/$/);
+              common.expectsError({
+                code: 'ERR_ASSERTION',
+                type: a.AssertionError,
+                message: /^\/a\/m deepEqual \/a\/$/
+              }));
 assert.throws(makeBlock(a.deepEqual, /a/igm, /a/im),
-              /^AssertionError: \/a\/gim deepEqual \/a\/im$/);
+              common.expectsError({
+                code: 'ERR_ASSERTION',
+                type: a.AssertionError,
+                message: /^\/a\/gim deepEqual \/a\/im$/
+              }));
 
 {
   const re1 = /a/g;
   re1.lastIndex = 3;
-
-  assert.throws(makeBlock(a.deepEqual, re1, /a/g),
-                /^AssertionError: \/a\/g deepEqual \/a\/g$/);
+  assert.doesNotThrow(makeBlock(a.deepEqual, re1, /a/g),
+                      common.expectsError({
+                        code: 'ERR_ASSERTION',
+                        message: /^\/a\/g deepEqual \/a\/g$/
+                      }));
 }
 
-
-// 7.4
 assert.doesNotThrow(makeBlock(a.deepEqual, 4, '4'), 'deepEqual(4, \'4\')');
 assert.doesNotThrow(makeBlock(a.deepEqual, true, 1), 'deepEqual(true, 1)');
 assert.throws(makeBlock(a.deepEqual, 4, '5'),
               a.AssertionError,
               'deepEqual( 4, \'5\')');
 
-// 7.5
 // having the same number of owned properties && the same set of keys
 assert.doesNotThrow(makeBlock(a.deepEqual, {a: 4}, {a: 4}));
 assert.doesNotThrow(makeBlock(a.deepEqual, {a: 4, b: '2'}, {a: 4, b: '2'}));
@@ -210,7 +248,6 @@ assert.doesNotThrow(
   'notDeepStrictEqual(new Date(), new Date(2000, 3, 14))'
 );
 
-// 7.3 - strict
 assert.doesNotThrow(makeBlock(a.deepStrictEqual, /a/, /a/));
 assert.doesNotThrow(makeBlock(a.deepStrictEqual, /a/g, /a/g));
 assert.doesNotThrow(makeBlock(a.deepStrictEqual, /a/i, /a/i));
@@ -218,33 +255,46 @@ assert.doesNotThrow(makeBlock(a.deepStrictEqual, /a/m, /a/m));
 assert.doesNotThrow(makeBlock(a.deepStrictEqual, /a/igm, /a/igm));
 assert.throws(
   makeBlock(a.deepStrictEqual, /ab/, /a/),
-  /^AssertionError: \/ab\/ deepStrictEqual \/a\/$/
-);
+  common.expectsError({
+    code: 'ERR_ASSERTION',
+    type: a.AssertionError,
+    message: /^\/ab\/ deepStrictEqual \/a\/$/
+  }));
 assert.throws(
   makeBlock(a.deepStrictEqual, /a/g, /a/),
-  /^AssertionError: \/a\/g deepStrictEqual \/a\/$/
-);
+  common.expectsError({
+    code: 'ERR_ASSERTION',
+    type: a.AssertionError,
+    message: /^\/a\/g deepStrictEqual \/a\/$/
+  }));
 assert.throws(
   makeBlock(a.deepStrictEqual, /a/i, /a/),
-  /^AssertionError: \/a\/i deepStrictEqual \/a\/$/
-);
+  common.expectsError({
+    code: 'ERR_ASSERTION',
+    type: a.AssertionError,
+    message: /^\/a\/i deepStrictEqual \/a\/$/
+  }));
 assert.throws(
   makeBlock(a.deepStrictEqual, /a/m, /a/),
-  /^AssertionError: \/a\/m deepStrictEqual \/a\/$/
-);
+  common.expectsError({
+    code: 'ERR_ASSERTION',
+    type: a.AssertionError,
+    message: /^\/a\/m deepStrictEqual \/a\/$/
+  }));
 assert.throws(
   makeBlock(a.deepStrictEqual, /a/igm, /a/im),
-  /^AssertionError: \/a\/gim deepStrictEqual \/a\/im$/
-);
+  common.expectsError({
+    code: 'ERR_ASSERTION',
+    type: a.AssertionError,
+    message: /^\/a\/gim deepStrictEqual \/a\/im$/
+  }));
 
 {
   const re1 = /a/;
   re1.lastIndex = 3;
-  assert.throws(makeBlock(a.deepStrictEqual, re1, /a/),
-                /^AssertionError: \/a\/ deepStrictEqual \/a\/$/);
+  assert.doesNotThrow(makeBlock(a.deepStrictEqual, re1, /a/));
 }
 
-// 7.4 - strict
 assert.throws(makeBlock(a.deepStrictEqual, 4, '4'),
               a.AssertionError,
               'deepStrictEqual(4, \'4\')');
@@ -257,18 +307,29 @@ assert.throws(makeBlock(a.deepStrictEqual, 4, '5'),
               a.AssertionError,
               'deepStrictEqual(4, \'5\')');
 
-// 7.5 - strict
 // having the same number of owned properties && the same set of keys
 assert.doesNotThrow(makeBlock(a.deepStrictEqual, {a: 4}, {a: 4}));
 assert.doesNotThrow(makeBlock(a.deepStrictEqual,
                               {a: 4, b: '2'},
                               {a: 4, b: '2'}));
 assert.throws(makeBlock(a.deepStrictEqual, [4], ['4']),
-              /^AssertionError: \[ 4 ] deepStrictEqual \[ '4' ]$/);
+              common.expectsError({
+                code: 'ERR_ASSERTION',
+                type: a.AssertionError,
+                message: /^\[ 4 ] deepStrictEqual \[ '4' ]$/
+              }));
 assert.throws(makeBlock(a.deepStrictEqual, {a: 4}, {a: 4, b: true}),
-              /^AssertionError: { a: 4 } deepStrictEqual { a: 4, b: true }$/);
+              common.expectsError({
+                code: 'ERR_ASSERTION',
+                type: a.AssertionError,
+                message: /^{ a: 4 } deepStrictEqual { a: 4, b: true }$/
+              }));
 assert.throws(makeBlock(a.deepStrictEqual, ['a'], {0: 'a'}),
-              /^AssertionError: \[ 'a' ] deepStrictEqual { '0': 'a' }$/);
+              common.expectsError({
+                code: 'ERR_ASSERTION',
+                type: a.AssertionError,
+                message: /^\[ 'a' ] deepStrictEqual { '0': 'a' }$/
+              }));
 //(although not necessarily the same order),
 assert.doesNotThrow(makeBlock(a.deepStrictEqual,
                               {a: 4, b: '1'},
@@ -338,7 +399,7 @@ assert.throws(makeBlock(a.deepStrictEqual, new Boolean(true), {}),
 
 // Testing the throwing
 function thrower(errorConstructor) {
-  throw new errorConstructor('test');
+  throw new errorConstructor({});
 }
 
 // the basic calls work
@@ -353,37 +414,44 @@ assert.throws(makeBlock(thrower, a.AssertionError));
 assert.throws(makeBlock(thrower, TypeError));
 
 // when passing a type, only catch errors of the appropriate type
-let threw = false;
-try {
-  a.throws(makeBlock(thrower, TypeError), a.AssertionError);
-} catch (e) {
-  threw = true;
-  assert.ok(e instanceof TypeError, 'type');
+{
+  let threw = false;
+  try {
+    a.throws(makeBlock(thrower, TypeError), a.AssertionError);
+  } catch (e) {
+    threw = true;
+    assert.ok(e instanceof TypeError, 'type');
+  }
+  assert.strictEqual(true, threw,
+                     'a.throws with an explicit error is eating extra errors',
+                     a.AssertionError);
 }
-assert.strictEqual(true, threw,
-                   'a.throws with an explicit error is eating extra errors',
-                   a.AssertionError);
-threw = false;
 
 // doesNotThrow should pass through all errors
-try {
-  a.doesNotThrow(makeBlock(thrower, TypeError), a.AssertionError);
-} catch (e) {
-  threw = true;
-  assert.ok(e instanceof TypeError);
+{
+  let threw = false;
+  try {
+    a.doesNotThrow(makeBlock(thrower, TypeError), a.AssertionError);
+  } catch (e) {
+    threw = true;
+    assert.ok(e instanceof TypeError);
+  }
+  assert.strictEqual(true, threw, 'a.doesNotThrow with an explicit error is ' +
+                     'eating extra errors');
 }
-assert.strictEqual(true, threw, 'a.doesNotThrow with an explicit error is ' +
-                   'eating extra errors');
 
 // key difference is that throwing our correct error makes an assertion error
-try {
-  a.doesNotThrow(makeBlock(thrower, TypeError), TypeError);
-} catch (e) {
-  threw = true;
-  assert.ok(e instanceof a.AssertionError);
+{
+  let threw = false;
+  try {
+    a.doesNotThrow(makeBlock(thrower, TypeError), TypeError);
+  } catch (e) {
+    threw = true;
+    assert.ok(e instanceof a.AssertionError);
+  }
+  assert.strictEqual(true, threw,
+                     'a.doesNotThrow is not catching type matching errors');
 }
-assert.strictEqual(true, threw,
-                   'a.doesNotThrow is not catching type matching errors');
 
 assert.throws(function() { assert.ifError(new Error('test error')); },
               /^Error: test error$/);
@@ -392,55 +460,59 @@ assert.doesNotThrow(function() { assert.ifError(); });
 
 assert.throws(() => {
   assert.doesNotThrow(makeBlock(thrower, Error), 'user message');
-}, /Got unwanted exception. user message/,
+}, /Got unwanted exception: user message/,
               'a.doesNotThrow ignores user message');
 
 // make sure that validating using constructor really works
-threw = false;
-try {
-  assert.throws(
-      function() {
-        throw ({});
-      },
-      Array
-  );
-} catch (e) {
-  threw = true;
+{
+  let threw = false;
+  try {
+    assert.throws(
+        function() {
+          throw ({}); // eslint-disable-line no-throw-literal
+        },
+        Array
+    );
+  } catch (e) {
+    threw = true;
+  }
+  assert.ok(threw, 'wrong constructor validation');
 }
-assert.ok(threw, 'wrong constructor validation');
 
 // use a RegExp to validate error message
-a.throws(makeBlock(thrower, TypeError), /test/);
+a.throws(makeBlock(thrower, TypeError), /\[object Object\]/);
 
 // use a fn to validate error object
 a.throws(makeBlock(thrower, TypeError), function(err) {
-  if ((err instanceof TypeError) && /test/.test(err)) {
+  if ((err instanceof TypeError) && /\[object Object\]/.test(err)) {
     return true;
   }
 });
 
 // https://github.com/nodejs/node/issues/3188
-threw = false;
+{
+  let threw = false;
 
-let AnotherErrorType;
-try {
-  const ES6Error = class extends Error {};
+  let AnotherErrorType;
+  try {
+    const ES6Error = class extends Error {};
 
-  AnotherErrorType = class extends Error {};
+    AnotherErrorType = class extends Error {};
 
-  const functionThatThrows = () => {
-    throw new AnotherErrorType('foo');
-  };
+    const functionThatThrows = () => {
+      throw new AnotherErrorType('foo');
+    };
 
-  assert.throws(functionThatThrows, ES6Error);
-} catch (e) {
-  threw = true;
-  assert(e instanceof AnotherErrorType,
-         `expected AnotherErrorType, received ${e}`);
+    assert.throws(functionThatThrows, ES6Error);
+  } catch (e) {
+    threw = true;
+    assert(e instanceof AnotherErrorType,
+           `expected AnotherErrorType, received ${e}`);
+  }
+
+  assert.ok(threw);
 }
 
-assert.ok(threw);
-
 // https://github.com/nodejs/node/issues/6416
 // Make sure circular refs don't throw.
 {
@@ -463,6 +535,18 @@ assert.ok(threw);
 
   a.throws(makeBlock(a.deepEqual, d, e), /AssertionError/);
   a.throws(makeBlock(a.deepStrictEqual, d, e), /AssertionError/);
+
+  // https://github.com/nodejs/node/issues/13314
+  const f = {};
+  f.ref = f;
+
+  const g = {};
+  g.ref = g;
+
+  const h = {ref: g};
+
+  a.throws(makeBlock(a.deepEqual, f, h), /AssertionError/);
+  a.throws(makeBlock(a.deepStrictEqual, f, h), /AssertionError/);
 }
 // GH-7178. Ensure reflexivity of deepEqual with `arguments` objects.
 const args = (function() { return arguments; })();
@@ -485,6 +569,38 @@ a.throws(makeBlock(a.deepEqual, args, []));
   a.doesNotThrow(makeBlock(a.deepEqual, someArgs, sameArgs));
 }
 
+// check messages from assert.throws()
+{
+  const noop = () => {};
+  assert.throws(
+    () => { a.throws((noop)); },
+    common.expectsError({
+      code: 'ERR_ASSERTION',
+      message: /^Missing expected exception\.$/
+    }));
+
+  assert.throws(
+    () => { a.throws(noop, TypeError); },
+    common.expectsError({
+      code: 'ERR_ASSERTION',
+      message: /^Missing expected exception \(TypeError\)\.$/
+    }));
+
+  assert.throws(
+    () => { a.throws(noop, 'fhqwhgads'); },
+    common.expectsError({
+      code: 'ERR_ASSERTION',
+      message: /^Missing expected exception: fhqwhgads$/
+    }));
+
+  assert.throws(
+    () => { a.throws(noop, TypeError, 'fhqwhgads'); },
+    common.expectsError({
+      code: 'ERR_ASSERTION',
+      message: /^Missing expected exception \(TypeError\): fhqwhgads$/
+    }));
+}
+
 const circular = {y: 1};
 circular.x = circular;
 
@@ -492,8 +608,8 @@ function testAssertionMessage(actual, expected) {
   try {
     assert.strictEqual(actual, '');
   } catch (e) {
-    assert.strictEqual(e.toString(),
-                       ['AssertionError:', expected, '===', '\'\''].join(' '));
+    assert.strictEqual(e.message,
+                       [expected, '===', '\'\''].join(' '));
     assert.ok(e.generatedMessage, 'Message not marked as generated');
   }
 }
@@ -522,74 +638,99 @@ testAssertionMessage({a: NaN, b: Infinity, c: -Infinity},
                      '{ a: NaN, b: Infinity, c: -Infinity }');
 
 // #2893
-try {
-  // eslint-disable-next-line assert-throws-arguments
-  assert.throws(function() {
-    assert.ifError(null);
-  });
-} catch (e) {
-  threw = true;
-  assert.strictEqual(e.message, 'Missing expected exception..');
+{
+  let threw = false;
+  try {
+    // eslint-disable-next-line assert-throws-arguments
+    assert.throws(function() {
+      assert.ifError(null);
+    });
+  } catch (e) {
+    threw = true;
+    assert.strictEqual(e.message, 'Missing expected exception.');
+  }
+  assert.ok(threw);
 }
-assert.ok(threw);
 
 // #5292
 try {
   assert.strictEqual(1, 2);
 } catch (e) {
-  assert.strictEqual(e.toString().split('\n')[0], 'AssertionError: 1 === 2');
+  assert.strictEqual(e.message.split('\n')[0], '1 === 2');
   assert.ok(e.generatedMessage, 'Message not marked as generated');
 }
 
 try {
   assert.strictEqual(1, 2, 'oh no');
 } catch (e) {
-  assert.strictEqual(e.toString().split('\n')[0], 'AssertionError: oh no');
+  assert.strictEqual(e.message.split('\n')[0], 'oh no');
   assert.strictEqual(e.generatedMessage, false,
                      'Message incorrectly marked as generated');
 }
 
-// Verify that throws() and doesNotThrow() throw on non-function block
-function testBlockTypeError(method, block) {
-  let threw = true;
+{
+  // Verify that throws() and doesNotThrow() throw on non-function block
+  const validationFunction = common.expectsError({
+    code: 'ERR_INVALID_ARG_TYPE',
+    type: TypeError
+  });
 
-  try {
-    method(block);
-    threw = false;
-  } catch (e) {
-    assert.strictEqual(e.toString(),
-                       'TypeError: "block" argument must be a function');
-  }
+  const testBlockTypeError = (method, block) => {
+    let threw = true;
 
-  assert.ok(threw);
+    try {
+      method(block);
+      threw = false;
+    } catch (e) {
+      validationFunction(e);
+    }
+
+    assert.ok(threw);
+  };
+
+  testBlockTypeError(assert.throws, 'string');
+  testBlockTypeError(assert.doesNotThrow, 'string');
+  testBlockTypeError(assert.throws, 1);
+  testBlockTypeError(assert.doesNotThrow, 1);
+  testBlockTypeError(assert.throws, true);
+  testBlockTypeError(assert.doesNotThrow, true);
+  testBlockTypeError(assert.throws, false);
+  testBlockTypeError(assert.doesNotThrow, false);
+  testBlockTypeError(assert.throws, []);
+  testBlockTypeError(assert.doesNotThrow, []);
+  testBlockTypeError(assert.throws, {});
+  testBlockTypeError(assert.doesNotThrow, {});
+  testBlockTypeError(assert.throws, /foo/);
+  testBlockTypeError(assert.doesNotThrow, /foo/);
+  testBlockTypeError(assert.throws, null);
+  testBlockTypeError(assert.doesNotThrow, null);
+  testBlockTypeError(assert.throws, undefined);
+  testBlockTypeError(assert.doesNotThrow, undefined);
 }
 
-testBlockTypeError(assert.throws, 'string');
-testBlockTypeError(assert.doesNotThrow, 'string');
-testBlockTypeError(assert.throws, 1);
-testBlockTypeError(assert.doesNotThrow, 1);
-testBlockTypeError(assert.throws, true);
-testBlockTypeError(assert.doesNotThrow, true);
-testBlockTypeError(assert.throws, false);
-testBlockTypeError(assert.doesNotThrow, false);
-testBlockTypeError(assert.throws, []);
-testBlockTypeError(assert.doesNotThrow, []);
-testBlockTypeError(assert.throws, {});
-testBlockTypeError(assert.doesNotThrow, {});
-testBlockTypeError(assert.throws, /foo/);
-testBlockTypeError(assert.doesNotThrow, /foo/);
-testBlockTypeError(assert.throws, null);
-testBlockTypeError(assert.doesNotThrow, null);
-testBlockTypeError(assert.throws, undefined);
-testBlockTypeError(assert.doesNotThrow, undefined);
-
 // https://github.com/nodejs/node/issues/3275
+// eslint-disable-next-line no-throw-literal
 assert.throws(() => { throw 'error'; }, (err) => err === 'error');
 assert.throws(() => { throw new Error(); }, (err) => err instanceof Error);
 
 // Long values should be truncated for display.
 assert.throws(() => {
   assert.strictEqual('A'.repeat(1000), '');
-}, new RegExp(`^AssertionError: '${'A'.repeat(127)} === ''$`));
+}, common.expectsError({
+  code: 'ERR_ASSERTION',
+  message: new RegExp(`^'${'A'.repeat(127)} === ''$`)}));
 
-console.log('All OK');
+{
+  // bad args to AssertionError constructor should throw TypeError
+  const args = [1, true, false, '', null, Infinity, Symbol('test'), undefined];
+  const re = /^The "options" argument must be of type object$/;
+  args.forEach((input) => {
+    assert.throws(
+      () => new assert.AssertionError(input),
+      common.expectsError({
+        code: 'ERR_INVALID_ARG_TYPE',
+        type: TypeError,
+        message: re
+      }));
+  });
+}
