@@ -22,10 +22,9 @@
 'use strict';
 const common = require('../common');
 
-if (!common.hasCrypto) {
+if (!common.hasCrypto)
   common.skip('missing crypto');
-  return;
-}
+
 const assert = require('assert');
 const crypto = require('crypto');
 
@@ -141,12 +140,12 @@ const expectedErrorRegexp = /^TypeError: size must be a number >= 0$/;
     new Uint8Array(new Array(10).fill(0))
   ];
   const errMessages = {
-    offsetNotNumber: /offset must be a number/,
-    offsetOutOfRange: /offset out of range/,
-    offsetNotUInt32: /offset must be a uint32/,
-    sizeNotNumber: /size must be a number/,
-    sizeNotUInt32: /size must be a uint32/,
-    bufferTooSmall: /buffer too small/,
+    offsetNotNumber: /^TypeError: offset must be a number$/,
+    offsetOutOfRange: /^RangeError: offset out of range$/,
+    offsetNotUInt32: /^TypeError: offset must be a uint32$/,
+    sizeNotNumber: /^TypeError: size must be a number$/,
+    sizeNotUInt32: /^TypeError: size must be a uint32$/,
+    bufferTooSmall: /^RangeError: buffer too small$/,
   };
 
   for (const buf of bufs) {
@@ -265,4 +264,4 @@ const expectedErrorRegexp = /^TypeError: size must be a number >= 0$/;
 // length exceeds max acceptable value"
 assert.throws(function() {
   crypto.randomBytes((-1 >>> 0) + 1);
-}, TypeError);
+}, /^TypeError: size must be a number >= 0$/);
