@@ -16,7 +16,7 @@ if (process.argv[2] === 'child') {
 }
 
 const filename = path.join(common.tmpDir, '/readfilesync_pipe_large_test.txt');
-const dataExpected = new Array(1000000).join('a');
+const dataExpected = 'a'.repeat(999999);
 common.refreshTmpDir();
 fs.writeFileSync(filename, dataExpected);
 
@@ -25,8 +25,7 @@ const f = JSON.stringify(__filename);
 const node = JSON.stringify(process.execPath);
 const cmd = `cat ${filename} | ${node} ${f} child`;
 exec(cmd, { maxBuffer: 1000000 }, function(err, stdout, stderr) {
-  if (err) console.error(err);
-  assert(!err, 'it exits normally');
+  assert.ifError(err);
   assert.strictEqual(stdout, dataExpected, 'it reads the file and outputs it');
   assert.strictEqual(stderr, '', 'it does not write to stderr');
   console.log('ok');
