@@ -1,16 +1,15 @@
 'use strict';
 const common = require('../common');
-const fixtures = require('../common/fixtures');
-
 if (!common.hasCrypto)
   common.skip('missing crypto');
 
 const assert = require('assert');
 const https = require('https');
+const fs = require('fs');
 
 const options = {
-  key: fixtures.readKey('agent1-key.pem'),
-  cert: fixtures.readKey('agent1-cert.pem')
+  key: fs.readFileSync(`${common.fixturesDir}/keys/agent1-key.pem`),
+  cert: fs.readFileSync(`${common.fixturesDir}/keys/agent1-cert.pem`)
 };
 const httpsServer = https.createServer(options, reqHandler);
 
@@ -50,9 +49,7 @@ function testHttps() {
 
   httpsServer.listen(0, function(er) {
     console.log(`test https server listening on port ${this.address().port}`);
-
-    if (er) throw er;
-
+    assert.ifError(er);
     https.get({
       method: 'GET',
       path: `/${counter++}`,
