@@ -1,3 +1,24 @@
+// Copyright Joyent, Inc. and other Node contributors.
+//
+// Permission is hereby granted, free of charge, to any person obtaining a
+// copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to permit
+// persons to whom the Software is furnished to do so, subject to the
+// following conditions:
+//
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
+// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
+// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
+// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
+// USE OR OTHER DEALINGS IN THE SOFTWARE.
+
 'use strict';
 require('../common');
 const assert = require('assert');
@@ -69,15 +90,15 @@ const unixSpecialCaseFormatTests = [
 
 const errors = [
   {method: 'parse', input: [null],
-   message: /^TypeError: Path must be a string. Received null$/},
+   message: /^TypeError: Path must be a string\. Received null$/},
   {method: 'parse', input: [{}],
-   message: /^TypeError: Path must be a string. Received {}$/},
+   message: /^TypeError: Path must be a string\. Received {}$/},
   {method: 'parse', input: [true],
-   message: /^TypeError: Path must be a string. Received true$/},
+   message: /^TypeError: Path must be a string\. Received true$/},
   {method: 'parse', input: [1],
-   message: /^TypeError: Path must be a string. Received 1$/},
+   message: /^TypeError: Path must be a string\. Received 1$/},
   {method: 'parse', input: [],
-   message: /^TypeError: Path must be a string. Received undefined$/},
+   message: /^TypeError: Path must be a string\. Received undefined$/},
   {method: 'format', input: [null],
    message:
       /^TypeError: Parameter "pathObject" must be an object, not object$/},
@@ -136,26 +157,22 @@ trailingTests.forEach(function(test) {
   test[1].forEach(function(test) {
     const actual = parse(test[0]);
     const expected = test[1];
-    const fn = `path.${os}.parse(`;
-    const message = fn +
-                    JSON.stringify(test[0]) +
-                    ')' +
-                    '\n  expect=' + JSON.stringify(expected) +
-                    '\n  actual=' + JSON.stringify(actual);
+    const message = `path.${os}.parse(${JSON.stringify(test[0])})\n  expect=${
+      JSON.stringify(expected)}\n  actual=${JSON.stringify(actual)}`;
     const actualKeys = Object.keys(actual);
     const expectedKeys = Object.keys(expected);
     let failed = (actualKeys.length !== expectedKeys.length);
     if (!failed) {
       for (let i = 0; i < actualKeys.length; ++i) {
         const key = actualKeys[i];
-        if (expectedKeys.indexOf(key) === -1 || actual[key] !== expected[key]) {
+        if (!expectedKeys.includes(key) || actual[key] !== expected[key]) {
           failed = true;
           break;
         }
       }
     }
     if (failed)
-      failures.push('\n' + message);
+      failures.push(`\n${message}`);
   });
 });
 assert.strictEqual(failures.length, 0, failures.join(''));
