@@ -4,23 +4,22 @@ const common = require('../common');
 const assert = require('assert');
 const path = require('path');
 const fs = require('fs');
-const { exec, spawn } = require('child_process');
-const fixtures = require('../common/fixtures');
+const exec = require('child_process').exec;
+const spawn = require('child_process').spawn;
+const util = require('util');
 
 common.refreshTmpDir();
 
-const linkTarget = fixtures.path('module-require-symlink',
-                                 'node_modules',
-                                 'dep2');
+const linkTarget = path.join(common.fixturesDir,
+                             '/module-require-symlink/node_modules/dep2/');
 
-const linkDir = fixtures.path('module-require-symlink',
-                              'node_modules',
-                              'dep1',
-                              'node_modules',
-                              'dep2');
+const linkDir = path.join(
+  common.fixturesDir,
+  '/module-require-symlink/node_modules/dep1/node_modules/dep2'
+);
 
-const linkScriptTarget = fixtures.path('module-require-symlink',
-                                       'symlinked.js');
+const linkScriptTarget = path.join(common.fixturesDir,
+                                   '/module-require-symlink/symlinked.js');
 
 const linkScript = path.join(common.tmpDir, 'module-require-symlink.js');
 
@@ -46,7 +45,8 @@ function test() {
   fs.symlinkSync(linkScriptTarget, linkScript);
 
   // load symlinked-module
-  const fooModule = require(fixtures.path('/module-require-symlink/foo.js'));
+  const fooModule =
+    require(path.join(common.fixturesDir, '/module-require-symlink/foo.js'));
   assert.strictEqual(fooModule.dep1.bar.version, 'CORRECT_VERSION');
   assert.strictEqual(fooModule.dep2.bar.version, 'CORRECT_VERSION');
 
@@ -54,7 +54,16 @@ function test() {
   const node = process.execPath;
   const child = spawn(node, ['--preserve-symlinks', linkScript]);
   child.on('close', function(code, signal) {
-    assert(!code);
+    assert.strictEqual(code, 0);
+    assert(!signal);
+  });
+
+  // Also verify that symlinks works for setting preserve via env variables
+  const childEnv = spawn(node, [linkScript], {
+    env: util._extend(process.env, {NODE_PRESERVE_SYMLINKS: '1'})
+  });
+  childEnv.on('close', function(code, signal) {
+    assert.strictEqual(code, 0);
     assert(!signal);
   });
 }
