@@ -1,3 +1,24 @@
+// Copyright Joyent, Inc. and other Node contributors.
+//
+// Permission is hereby granted, free of charge, to any person obtaining a
+// copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to permit
+// persons to whom the Software is furnished to do so, subject to the
+// following conditions:
+//
+// The above copyright notice and this permission notice shall be included
+// in all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
+// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
+// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
+// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
+// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
+// USE OR OTHER DEALINGS IN THE SOFTWARE.
+
 'use strict';
 const common = require('../common');
 
@@ -19,8 +40,9 @@ const assert = require('assert');
 const { spawn } = require('child_process');
 const { SSL_OP_NO_SESSION_RESUMPTION_ON_RENEGOTIATION } =
   require('crypto').constants;
+const fs = require('fs');
+const path = require('path');
 const tls = require('tls');
-const fixtures = require('../common/fixtures');
 
 const testCases =
   [{ title: 'Do not request certs. Everyone is unauthorized.',
@@ -30,88 +52,90 @@ const testCases =
      CAs: ['ca1-cert'],
      clients:
      [{ name: 'agent1', shouldReject: false, shouldAuth: false },
-      { name: 'agent2', shouldReject: false, shouldAuth: false },
-      { name: 'agent3', shouldReject: false, shouldAuth: false },
-      { name: 'nocert', shouldReject: false, shouldAuth: false }
+        { name: 'agent2', shouldReject: false, shouldAuth: false },
+        { name: 'agent3', shouldReject: false, shouldAuth: false },
+        { name: 'nocert', shouldReject: false, shouldAuth: false }
      ]
   },
 
-   { title: 'Allow both authed and unauthed connections with CA1',
-     requestCert: true,
-     rejectUnauthorized: false,
-     renegotiate: false,
-     CAs: ['ca1-cert'],
-     clients:
+  { title: 'Allow both authed and unauthed connections with CA1',
+    requestCert: true,
+    rejectUnauthorized: false,
+    renegotiate: false,
+    CAs: ['ca1-cert'],
+    clients:
     [{ name: 'agent1', shouldReject: false, shouldAuth: true },
-     { name: 'agent2', shouldReject: false, shouldAuth: false },
-     { name: 'agent3', shouldReject: false, shouldAuth: false },
-     { name: 'nocert', shouldReject: false, shouldAuth: false }
+        { name: 'agent2', shouldReject: false, shouldAuth: false },
+        { name: 'agent3', shouldReject: false, shouldAuth: false },
+        { name: 'nocert', shouldReject: false, shouldAuth: false }
     ]
-   },
+  },
 
-   { title: 'Do not request certs at connection. Do that later',
-     requestCert: false,
-     rejectUnauthorized: false,
-     renegotiate: true,
-     CAs: ['ca1-cert'],
-     clients:
+  { title: 'Do not request certs at connection. Do that later',
+    requestCert: false,
+    rejectUnauthorized: false,
+    renegotiate: true,
+    CAs: ['ca1-cert'],
+    clients:
     [{ name: 'agent1', shouldReject: false, shouldAuth: true },
-     { name: 'agent2', shouldReject: false, shouldAuth: false },
-     { name: 'agent3', shouldReject: false, shouldAuth: false },
-     { name: 'nocert', shouldReject: false, shouldAuth: false }
+        { name: 'agent2', shouldReject: false, shouldAuth: false },
+        { name: 'agent3', shouldReject: false, shouldAuth: false },
+        { name: 'nocert', shouldReject: false, shouldAuth: false }
     ]
-   },
+  },
 
-   { title: 'Allow only authed connections with CA1',
-     requestCert: true,
-     rejectUnauthorized: true,
-     renegotiate: false,
-     CAs: ['ca1-cert'],
-     clients:
+  { title: 'Allow only authed connections with CA1',
+    requestCert: true,
+    rejectUnauthorized: true,
+    renegotiate: false,
+    CAs: ['ca1-cert'],
+    clients:
     [{ name: 'agent1', shouldReject: false, shouldAuth: true },
-     { name: 'agent2', shouldReject: true },
-     { name: 'agent3', shouldReject: true },
-     { name: 'nocert', shouldReject: true }
+        { name: 'agent2', shouldReject: true },
+        { name: 'agent3', shouldReject: true },
+        { name: 'nocert', shouldReject: true }
     ]
-   },
+  },
 
-   { title: 'Allow only authed connections with CA1 and CA2',
-     requestCert: true,
-     rejectUnauthorized: true,
-     renegotiate: false,
-     CAs: ['ca1-cert', 'ca2-cert'],
-     clients:
+  { title: 'Allow only authed connections with CA1 and CA2',
+    requestCert: true,
+    rejectUnauthorized: true,
+    renegotiate: false,
+    CAs: ['ca1-cert', 'ca2-cert'],
+    clients:
     [{ name: 'agent1', shouldReject: false, shouldAuth: true },
-     { name: 'agent2', shouldReject: true },
-     { name: 'agent3', shouldReject: false, shouldAuth: true },
-     { name: 'nocert', shouldReject: true }
+        { name: 'agent2', shouldReject: true },
+        { name: 'agent3', shouldReject: false, shouldAuth: true },
+        { name: 'nocert', shouldReject: true }
     ]
-   },
+  },
 
 
-   { title: 'Allow only certs signed by CA2 but not in the CRL',
-     requestCert: true,
-     rejectUnauthorized: true,
-     renegotiate: false,
-     CAs: ['ca2-cert'],
-     crl: 'ca2-crl',
-     clients: [
-       { name: 'agent1', shouldReject: true, shouldAuth: false },
-       { name: 'agent2', shouldReject: true, shouldAuth: false },
-       { name: 'agent3', shouldReject: false, shouldAuth: true },
-       // Agent4 has a cert in the CRL.
-       { name: 'agent4', shouldReject: true, shouldAuth: false },
-       { name: 'nocert', shouldReject: true }
-     ]
-   }
+  { title: 'Allow only certs signed by CA2 but not in the CRL',
+    requestCert: true,
+    rejectUnauthorized: true,
+    renegotiate: false,
+    CAs: ['ca2-cert'],
+    crl: 'ca2-crl',
+    clients: [
+        { name: 'agent1', shouldReject: true, shouldAuth: false },
+        { name: 'agent2', shouldReject: true, shouldAuth: false },
+        { name: 'agent3', shouldReject: false, shouldAuth: true },
+        // Agent4 has a cert in the CRL.
+        { name: 'agent4', shouldReject: true, shouldAuth: false },
+        { name: 'nocert', shouldReject: true }
+    ]
+  }
   ];
 
+
 function filenamePEM(n) {
-  return fixtures.path('keys', `${n}.pem`);
+  return path.join(common.fixturesDir, 'keys', `${n}.pem`);
 }
 
+
 function loadPEM(n) {
-  return fixtures.readKey(`${n}.pem`);
+  return fs.readFileSync(filenamePEM(n));
 }
 
 
@@ -223,7 +247,7 @@ function runClient(prefix, port, options, cb) {
       assert.strictEqual(
         options.shouldAuth, authed,
         `${prefix}${options.name} authed is ${authed} but should have been ${
-          options.shouldAuth}`);
+        options.shouldAuth}`);
     }
 
     cb();
@@ -238,7 +262,7 @@ function runTest(port, testIndex) {
   const tcase = testCases[testIndex];
   if (!tcase) return;
 
-  console.error(`${prefix}Running '${tcase.title}'`);
+  console.error(`${prefix}Running '%s'`, tcase.title);
 
   const cas = tcase.CAs.map(loadPEM);
 
@@ -265,8 +289,8 @@ function runTest(port, testIndex) {
   let renegotiated = false;
   const server = tls.Server(serverOptions, function handleConnection(c) {
     c.on('error', function(e) {
-      // child.kill() leads ECONNRESET error in the TLS connection of
-      // openssl s_client via spawn(). A test result is already
+      // child.kill() leads ECONNRESET errro in the TLS connection of
+      // openssl s_client via spawn(). A Test result is already
       // checked by the data of client.stdout before child.kill() so
       // these tls errors can be ignored.
     });
@@ -279,7 +303,7 @@ function runTest(port, testIndex) {
           requestCert: true,
           rejectUnauthorized: false
         }, function(err) {
-          assert(!err);
+          assert.ifError(err);
           c.write('\n_renegotiated\n');
           handleConnection(c);
         });
@@ -289,7 +313,7 @@ function runTest(port, testIndex) {
 
     if (c.authorized) {
       console.error(`${prefix}- authed connection: ${
-        c.getPeerCertificate().subject.CN}`);
+                    c.getPeerCertificate().subject.CN}`);
       c.write('\n_authed\n');
     } else {
       console.error(`${prefix}- unauthed connection: %s`, c.authorizationError);
@@ -306,7 +330,7 @@ function runTest(port, testIndex) {
     } else {
       server.close();
       successfulTests++;
-      runTest(0, nextTest++);
+      runTest(port, nextTest++);
     }
   }
 
@@ -325,7 +349,7 @@ function runTest(port, testIndex) {
             if (clientsCompleted === tcase.clients.length) {
               server.close();
               successfulTests++;
-              runTest(0, nextTest++);
+              runTest(port, nextTest++);
             }
           });
         }
@@ -337,6 +361,7 @@ function runTest(port, testIndex) {
 
 let nextTest = 0;
 runTest(0, nextTest++);
+runTest(0, nextTest++);
 
 
 process.on('exit', function() {
