@@ -1,10 +1,8 @@
 'use strict';
 const common = require('../common');
 
-if (!common.hasIntl) {
+if (!common.hasIntl)
   common.skip('missing Intl');
-  return;
-}
 
 const assert = require('assert');
 const { domainToASCII, domainToUnicode } = require('url');
