@@ -1,6 +1,11 @@
 'use strict';
 
 const common = require('../common');
+if (!common.hasIntl) {
+  // A handful of the tests fail when ICU is not included.
+  common.skip('missing Intl');
+}
+
 const assert = require('assert');
 const path = require('path');
 const URL = require('url').URL;
@@ -8,12 +13,6 @@ const { test, assert_equals } = require('../common/wpt');
 const additionalTestCases = require(
     path.join(common.fixturesDir, 'url-setter-tests-additional.js'));
 
-if (!common.hasIntl) {
-  // A handful of the tests fail when ICU is not included.
-  common.skip('missing Intl');
-  return;
-}
-
 const request = {
   response: require(path.join(common.fixturesDir, 'url-setter-tests'))
 };
@@ -119,10 +118,7 @@ startURLSettersTests()
                     /^Error: toString$/,
                     `url.${name} = { toString() { throw ... } }`);
       assert.throws(() => url[name] = sym,
-                    common.engineSpecificMessage({
-                      v8: /^TypeError: Cannot convert a Symbol value to a string$/, // eslint-disable-line max-len
-                      chakracore: /^TypeError: Object doesn't support property or method 'ToString'$/, // eslint-disable-line max-len
-                    }),
+                    /^TypeError: Cannot convert a Symbol value to a string$/,
                     `url.${name} = ${String(sym)}`);
     }
   }
